/**
 * Copyright (c) 2021 Noel <cutie@floofy.dev>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

import gay.floof.utils.gradle.configurePom
import java.util.Properties

buildscript {
    repositories {
        mavenLocal()
        maven {
            url = uri("https://plugins.gradle.org/m2/")
        }
    }

    dependencies {
        classpath("com.diffplug.spotless:spotless-plugin-gradle:6.1.2")
        classpath("org.jetbrains.kotlin:kotlin-gradle-plugin:1.6.10")
        classpath("org.jetbrains.dokka:dokka-gradle-plugin:1.6.10")
        classpath("org.jetbrains.dokka:dokka-base:1.6.10")
        classpath("io.kotest:kotest-gradle-plugin:0.3.9")
    }
}

plugins {
    id("com.diffplug.spotless") version "6.1.2"
    id("org.jetbrains.dokka") version "1.6.10"
    id("io.kotest") version "0.3.9"
    kotlin("jvm") version "1.6.10"
    `maven-publish`
}

group = "gay.floof"
version = "1.3.0"

repositories {
    mavenCentral()
    mavenLocal()
}

// Setup root project tasks
tasks {
    val dokkaOutput = "${rootProject.projectDir}/docs"

    clean {
        delete(dokkaOutput)
    }

    dokkaHtmlMultiModule.configure {
        dependsOn(clean)
        outputDirectory.set(file(dokkaOutput))
    }
}

println("using Java v${JavaVersion.current()}, Kotlin v${KotlinVersion.CURRENT}")

subprojects {
    // Define project metadata
    group = "gay.floof.utils"
    version = rootProject.version as String

    // Define plugins from root project
    apply(plugin = "kotlin")
    apply(plugin = "java-library")
    apply(plugin = "maven-publish")
    apply(plugin = "org.jetbrains.dokka")
    apply(plugin = "com.diffplug.spotless")

    // Setup Spotless
    spotless {
        kotlin {
            trimTrailingWhitespace()
            endWithNewline()
            licenseHeaderFile("${rootProject.projectDir}/assets/HEADING")

            // We can't use the .editorconfig file, so we'll have to specify it here
            // issue: https://github.com/diffplug/spotless/issues/142
            // ktlint 0.35.0 (default for Spotless) doesn't support trailing commas
            ktlint("0.43.0")
                .userData(mapOf(
                    "no-consecutive-blank-lines" to "true",
                    "no-unit-return" to "true",
                    "disabled_rules" to "no-wildcard-imports,colon-spacing",
                    "indent_size" to "4"
                ))
        }
    }

    // Add repositories and dependencies for all subprojects
    repositories {
        mavenCentral()
        mavenLocal()
        jcenter()
    }

    dependencies {
        // Kotlin stdlib
        implementation(kotlin("stdlib", "1.6.10"))

        // Testing utilities
        testImplementation(kotlin("test"))
    }

    tasks.test {
        useJUnitPlatform()
    }

    // Setup Dokka tasks
    tasks.dokkaHtml.configure {
        outputDirectory.set(file("${rootProject.projectDir}/docs"))
        dokkaSourceSets {
            configureEach {
                platform.set(org.jetbrains.dokka.Platform.jvm)
                sourceLink {
                    localDirectory.set(file("src/main/kotlin"))
                    remoteUrl.set(uri("https://github.com/auguwu/common-utils/tree/master/${project.name}/src/main/kotlin").toURL())
                    remoteLineSuffix.set("#L")
                }

                jdkVersion.set(8)
            }
        }
    }

    // In order to use @OptIn, we need to add the compiler flag
    tasks.compileKotlin.configure {
        // Why 1.8 support?
        //
        // > Task :slf4j:compileTestKotlin FAILED
        // e: D:\Projects\Libraries\Kotlin\commons\slf4j\src\test\kotlin\Slf4jTests.kt: (32, 16): Cannot inline bytecode built with JVM target 16 into bytecode that is being built with JVM target
        // 1.8. Please specify proper '-jvm-target' option
        kotlinOptions.jvmTarget = JavaVersion.VERSION_1_8.toString()
        kotlinOptions.javaParameters = true
        kotlinOptions.freeCompilerArgs += listOf(
            "-Xopt-in=kotlin.RequiresOptIn"
        )
    }

    // Gradle cannot add `-D...` for some reason?
    val s3Endpoint: Any? = findProperty("org.gradle.s3.endpoint") ?: System.getenv("org.gradle.s3.endpoint")
    if (s3Endpoint == null) {
        System.setProperty("org.gradle.s3.endpoint", "https://s3.wasabisys.com")
    }

    // Setup publishing to post to maven.floofy.dev
    val publishingProps = try {
        Properties().apply { load(file("${rootProject.projectDir}/gradle/publishing.properties").reader()) }
    } catch(e: Exception) {
        Properties()
    }

    val sourcesJar by tasks.registering(Jar::class) {
        archiveClassifier.set("sources")
        from(sourceSets.main.get().allSource)
    }

    val dokkaJar by tasks.registering(Jar::class) {
        group = JavaBasePlugin.DOCUMENTATION_GROUP
        description = "Assemble Kotlin documentation with Dokka"

        archiveClassifier.set("javadoc")
        from(tasks.dokkaHtml)
        dependsOn(tasks.dokkaHtml)
    }

    publishing {
        publications {
            create<MavenPublication>("commons") {
                from(components["kotlin"])

                groupId = "gay.floof.commons"
                artifactId = "commons-${project.name}"
                version = project.version as String

                artifact(sourcesJar.get())
                artifact(dokkaJar.get())

                pom {
                    configurePom(project.name)
                }
            }
        }

        repositories {
            maven(url = "s3://maven.floofy.dev/repo/releases") {
                credentials(AwsCredentials::class.java) {
                    accessKey = publishingProps.getProperty("s3.accessKey") ?: ""
                    secretKey = publishingProps.getProperty("s3.secretKey") ?: ""
                }
            }
        }
    }
}
